classdef Polar
    % POLAR  Class for polars and their manipulations.
    %   The POLAR class is used to represent completely one or multiple polar curves related to a
    %   single airfoil.
    %
    %   The class also implements quick shortcuts for the more generic functions of af_tools, such
    %   as findcllinearrange, findstall or findzerolift.
    %
    % <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
    % <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
    % -----
    %
    % Properties:
    %   - airfoil  : Airfoil name
    %   - reynolds : Rynolds number of each polar curve
    %   - mach     : Mach number of each polar curve
    %   - nCrit    : nCrit number of each polar curve
    %   - aoa      : Angles of attack (one column per Reynolds)
    %   - cl       : Lift coefficient (one row per aoa, one column per Reynolds)
    %   - cd       : Drag coefficient (one row per aoa, one column per Reynolds)
    %   - cm       : Moment coefficient (one row per aoa, one column per Reynolds)
    %
    % Private properties (set by running the methods below):
    %   - Ext   : Struct with the data of the extended polars
    %   - Stall : Struct with the data of the stall point for each curve
    %   - Zero  : Struct with the data of the zero-lift point for each curve
    %   - Lin   : Struct with the data of the linear range each curve
    %
    % Methods (consult the help of each method with "help af_tools.Polar.<method>"):
    %   - loadpolar      : Load a polar from the MAT-file
    %   - extendpolar    : Extend polar curves over [-pi, pi]
    %   - findstall      : Find stall point
    %   - findzerolift   : Find zero-lift point
    %   - findcllinrange : Find lift curve linear range
    %   - analyse        : Runs the above three functions
    %   - plotpolars     : Plot the polars
    %
    %
    % See also: af_tools.xf2mat.
    %
    % -----
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % Apache 2.0 License
    % https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

    % ----------------------------------------------------------------------------------------------

    properties
        airfoil  (1, :) cell
        reynolds (1, :) double {mustBeNonnegative}
        mach     (1, :) double {mustBeNonnegative}
        nCrit    (1, :) double {mustBeNonnegative}
        aoa      (:, :) double {mustBeReal}
        cl       (:, :) double {mustBeReal}
        cd       (:, :) double {mustBeReal}
        cm       (:, :) double {mustBeReal}
    end

    properties (SetAccess = private)
        Ext   = struct('aoa', [], 'cl', [], 'cd', [], 'cm', [])
        Stall = struct('aoa', [], 'cl', [], 'cd', [])
        Zero  = struct('aoa', [], 'cd', [])
        Lin   = struct('slope', [], 'aoaRange', [],  'clRange', [])
    end

    % ----------------------------------------------------------------------------------------------
    methods

        function self = Polar(reynolds, aoa, c_l, c_d, c_m)
            % POLAR Constructor for Polar object
            %   Create a polar object with basic information.
            %
            % See also: af_tools.Polar.loadpolar

            if nargin > 0
                narginchk(3, 5);

                self.reynolds = reynolds;
                self.aoa = aoa;
                self.cl = c_l;
                if nargin >= 4
                    self.cd = c_d;
                    if nargin >= 5
                        self.cm = c_m;
                    end
                end
            end

        end

        % ---------------------------------------
        function self = extendpolar(self)
            % EXTENDPOLAR  Extends the polar data over the full range of AOA ([-180, 180] deg)
            %
            % See also: af_tools.extendpolar.
            [self.Ext.aoa, self.Ext.cl, self.Ext.cd] = af_tools.extendpolar(self);
        end

        % ---------------------------------------
        function self = findstall(self)
            % FINDSTALL  Find the stall point of the polars
            %
            % See also: af_tools.findstall.

            [self.Stall.aoa, self.Stall.cl, self.Stall.cd] = af_tools.findstall(self);
        end

        % ---------------------------------------
        function self = findzerolift(self)
            % FINDZEROLIFT  Find the zero-lift angle and associated cd
            %
            % See also: af_tools.findzerolift.

            [self.Zero.aoa, self.Zero.cd] = af_tools.findzerolift(self);
        end

        % ---------------------------------------
        function self = findcllinrange(self)
            % FINDZEROLIFT  Find the linear range of the cl and its slope
            %
            % See also: af_tools.findcllinrange.

            [self.Lin.clRange, self.Lin.aoaRange, self.Lin.slope] = ...
                af_tools.findcllinearrange(self);
        end

        % ---------------------------------------
        function plotpolars(self)
            % PLOTPOLARS  Plot the polars
            %
            % See also: af_tools.plotpolars.

            af_tools.plotpolars(self);
        end

        % ---------------------------------------
        function self = analyse(self)
            % ANALYSE  Get all useful information from the polar curves at once
            %
            % See also: af_tools.findstall, af_tools.findzerolift, af_tools.findcllinrange.

            self = findstall(self);
            self = findzerolift(self);
            self = findcllinrange(self);
        end

    end

    % ----------------------------------------------------------------------------------------------

    methods (Static)
        self = loadpolar(polarFile) % Load polar from file
    end
end
