function obj = loadpolar(polarFile)
    % LOADPOLAR  Load a polar object from a file
    %   This static method can be used in place of a normal constructor.
    %   When this function is used, the Polar object saved in the MAT-File will be imported. By
    %   assigning the output to something, it effectively duplicates the polar from the MAT-file.
    % -----
    %
    % Usage:
    %   Polar.LOADPOLAR(polarFile) loads the polar object stored in polarFile into the NewPolar.
    %
    % Inputs:
    %   polarFile : MAT-File containing only a single polar object.
    %
    % Output:
    %   Polar object contained in polarFile.
    %
    % Example:
    %   NewPolar = af_tools.Polar.loadpolar('oldpolar.mat');
    %
    % See also: af_tools.Polar.
    %
    % -----
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % Apache 2.0 License
    % https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

    % ----------------------------------------------------------------------------------------------

    % Load the file
    tmp = load(polarFile);

    % Get name of variables in file (normally only polar object)
    polarName = fieldnames(tmp);

    % Check for issues and assign polar object
    if isempty(polarName)
        error('POLAR:loadpolar:emptyPolarFile', ...
              ['The Polar file specified as input does not contain anything!\n'...
               'See <help af_tools.Polar.loadpolar> for details about this method.']);
    elseif length(polarName) > 1
        error('POLAR:loadpolar:tooManyVarInPolarFile', ...
              ['The Polar file specified as input should only contain '...
               'one polar object and nothing else.\n'...
               'Found %d different variables instead!\n'...
               'See <help af_tools.Polar.loadpolar> for details about this method.'], ...
              length(polarName));
    else
        obj = tmp.(polarName{1});
    end

end
