% TEST_SAVETODAT Unitary tests for the savetodat sub-function
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.
%
% -----
% (c) Copyright 2022 University of Liege
% Author: Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

% --------------------------------------------------------------------------------------------------

%% Main test function

function tests = test_savetodat

    tests = functiontests(localfunctions);

end

%% Setup and teardown

function setupOnce(testCase)

    addpath('../.'); % Add repository to Matlab Path
    addpath('./test_utils'); % Add utils to Matlab Path

    % Set random number generator settings
    testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

    rmpath('../.'); % Remove repository from Matlab Path
    rmpath('./test_utils'); % Remove utils from Matlab Path

    % Restore the random number generator settings
    s = testCase.TestData.currentRNG;
    rng(s);

end

function teardown(~)

    close all; % Close all figures that would have been openend

end

%% Test for correct output

function test_saveArray(testCase)
    % Save a single array as is

    filename = 'testDatArray.dat';
    header = 'test-array';
    array = rand(10, 3);

    af_tools.utils.savetodat(filename, header, array);
    [headFile, arrFile] = loaddat3col(filename);
    headFile = strtrim(char(headFile{:}));

    verifyEqual(testCase, headFile, header);
    verifyEqual(testCase, arrFile, array, 'AbsTol', 1e-7);

    % Cleanup
    delete(filename);

end

function test_saveVectors(testCase)
    % Save multiple vectors of the same size as a single array

    filename = 'testDatVector.dat';
    header = 'test-vectors';
    vect1 = rand(10, 1);
    vect2 = rand(1, 10);
    vect3 = 1:10;

    af_tools.utils.savetodat(filename, header, vect1, vect2, vect3);
    [headFile, arrFile] = loaddat3col(filename);
    headFile = strtrim(char(headFile{:}));

    vect1 = af_tools.utils.vecttocol(vect1);
    vect2 = af_tools.utils.vecttocol(vect2);
    vect3 = af_tools.utils.vecttocol(vect3);

    verifyEqual(testCase, headFile, header);
    verifyEqual(testCase, arrFile, [vect1, vect2, vect3], 'AbsTol', 1e-7);

    % Cleanup
    delete(filename);

end

% -------------------------------------
function [header, array] = loaddat3col(filename)

    % LOADDAT Loads dat-file and split header and array
    fileID = fopen(fullfile(pwd, filename));
    header = textscan(fileID, '%s', 1, 'Delimiter', '\n:', 'HeaderLines', 0);
    array = cell2mat(textscan(fileID, '%f %f %f %*[^\n]', 'HeaderLines', 0));

    fclose(fileID);
    clear fileID;
end
